package com.dataflux.architect.example;

import com.dataflux.architect.ISupportTimings;
import com.dataflux.architect.SimpleStepAdapter;
import com.dataflux.architect.StepException;
import com.dataflux.architect.Timings;

/**
 * Pass the input values directly as output values.
 * <p>
 * This is an example implementation of {@link com.dataflux.architect.ISimpleStep} extending
 * the {@link com.dataflux.architect.SimpleStepAdapter} abstract class.
 * </p>
 * <p>
 * By default, the {@link com.dataflux.architect.SimpleStepAdapter} abstract class
 * passes the values of the input fields directly to the output.  The input values may
 * be inspected by using a Java\u2122 debugger and placing a breakpoint on the
 * {@link #processRow()} method.
 * </p>
 * <p>
 * <strong>Note:</strong> The plugin examples are included in the Data Management Platform Java&#x2122;
 * Plugin jar and as such have a package name that represents that location.  Therefore, modifying the
 * examples requires placing the modified source, including the package statement, in a new Java&#x2122;
 * compilation unit.
 * </p>
 *
 * @author <a href="http://www.sas.com">SAS Institute</a>
 * @since dfPower Studio 8.0
 */
public class DebugSimpleStep
       extends SimpleStepAdapter
       implements ISupportTimings
{
   /** Initially set Timings instance. */
   private Timings mSavedTimings = null;

   /** Emit timings prefix. */
   private final String mTimingsPrefix;

   public DebugSimpleStep()
   {
      mTimingsPrefix = this.getClass().getSimpleName();
   }

   public int getParameterCount()
   {
      return 1;
   }

   public String getParameterName(int index)
          throws IndexOutOfBoundsException
   {
      if (0 == index)
         return Timings.TIMINGS_PARAMETER;
      else
         throw new IndexOutOfBoundsException();
   }

   public void processRow()
          throws StepException
   {
      long startTime = 0;

      if (null != mTimings)
         startTime = System.currentTimeMillis();

      super.processRow();

      if (null != mTimings)
         mTimings.updateTimings(mTimingsPrefix + ".processRow", System.currentTimeMillis() - startTime);
   }

   public void setParameterValue(int index, String value)
          throws IndexOutOfBoundsException
   {
      if (0 == index)
         {
         Boolean bool = new Boolean(value);
         if (bool.booleanValue())
            {
            mTimings = mSavedTimings;
            mTimings.enableTimings();
            }
         }
      else
         throw new IndexOutOfBoundsException();
   }

   public void setTimings(Timings timings)
   {
      mSavedTimings = timings;
   }
}
