package com.dataflux.architect.example;

import com.dataflux.architect.ISupportTimings;
import com.dataflux.architect.PrepState;
import com.dataflux.architect.StepAdapter;
import com.dataflux.architect.StepOutputItem;
import com.dataflux.architect.StepOutputItems;
import com.dataflux.architect.StepReadState;
import com.dataflux.architect.Timings;
import java.util.HashMap;

/**
 * Pass the input values directly as output values.
 * <p>
 * This is an example implementation of {@link com.dataflux.architect.IStep} extending
 * the {@link com.dataflux.architect.StepAdapter} abstract class.
 * </p>
 * <p>
 * <strong>Note:</strong> The plugin examples are included in the Data Management Platform Java&#x2122;
 * Plugin jar and as such have a package name that represents that location.  Therefore, modifying the
 * examples requires placing the modified source, including the package statement, in a new Java&#x2122;
 * compilation unit.
 * </p>
 *
 * @author <a href="http://www.sas.com">SAS Institute</a>
 * @since Data Management Platform 2.5
 */
public class DebugStep
       extends StepAdapter
       implements ISupportTimings
{
   /** Initially set Timings instance. */
   private Timings mSavedTimings = null;

   /** Emit timings prefix. */
   private final String mTimingsPrefix;

   public DebugStep()
   {
      mParameterValues   = new HashMap<String, String>();
      mParameterNames    = new String[1];
      mParameterNames[0] = Timings.TIMINGS_PARAMETER;
      mTimingsPrefix     = this.getClass().getSimpleName();
   }

   public StepReadState next()
   {
      int             count;
      long            startTime = 0;
      StepOutputItems ourItems,
                      parentItems;
      StepReadState   readState = StepReadState.ERROR;

      try
         {
         if (null != mTimings)
            startTime = System.currentTimeMillis();

         // Start with a parent row
         readState = step.parentNext();
         if (StepReadState.SUCCESS == readState)
            {
            parentItems = step.parentOutputItems(true);
            count       = parentItems.getCount();
            ourItems    = step.getExistingOutputItems();
            for (int i = 0; i < count; i++)
               ourItems.getItem(i).setValue(parentItems.getItem(i).getValue());

            mNumRowsRead++;
            }
         }
      catch (Throwable th)
         {
         step.setError("Error: " + th.getLocalizedMessage());
         readState = StepReadState.ERROR;
         }

      if (null != mTimings)
         mTimings.updateTimings(mTimingsPrefix + ".next", System.currentTimeMillis() - startTime);

      return readState;
   }

   public boolean preExec()
   {
      boolean result    = true;
      long    startTime = 0;

      super.preExec();

      if (null != mTimings)
         startTime = System.currentTimeMillis();

      // This example requires a parent step
      if (0 == step.getStep().getNumParents())
         {
         step.setError(this.getClass().getSimpleName() + " requires a parent step.");
         result = false;
         }

      if (null != mTimings)
         mTimings.updateTimings(mTimingsPrefix + ".preExec", System.currentTimeMillis() - startTime);

      return result;
   }

   public PrepState prep(int flags)
   {
      Boolean        bool;
      int            count,
                     i;
      long           startTime = 0;
      StepOutputItem item,
                     outItem;
      
      bool = new Boolean(mParameterValues.get(Timings.TIMINGS_PARAMETER));
      
      if (bool.booleanValue())
         {
         mTimings  = mSavedTimings;
         mTimings.enableTimings();
         startTime = System.currentTimeMillis();
         }

      // We're an echo box, therefore we want all input columns
      step.clearInputNames();
      StepOutputItems parentItems = step.parentOutputItems(flags == PREP_FLAG_FULL);
      if (null != parentItems)
         {
         count = parentItems.getCount();

         // Locate the field to process and remember its item index
         for (i = 0; i < count; i++)
            {
            item = parentItems.getItem(i);
            step.addInputName(item.getName());
            }
         }

      // If a quick prep(), then there is no need to return our output fields.
      if (flags == PREP_FLAG_QUICK)
         {
         if (null != mTimings)
            mTimings.updateTimings(mTimingsPrefix + ".prep", System.currentTimeMillis() - startTime);

         return PrepState.INPUTNAMES;
         }

      // Again, we want all input items in our output
      if (null != parentItems)
         {
         count                   = parentItems.getCount();
         StepOutputItems myItems = step.buildOutputItems(count);
         for (i = 0; i < count; i++)
            {
            item    = parentItems.getItem(i); 
            outItem = myItems.getItem(i);
            outItem.setMaxByteLength(item.getMaxByteLength());
            outItem.setMaxCharLength(item.getMaxCharLength());
            outItem.setType(item.getType());
            outItem.setName(item.getName());
            }
         }

      if (null != mTimings)
         mTimings.updateTimings(mTimingsPrefix + ".prep", System.currentTimeMillis() - startTime);

      return PrepState.ALL_SUCCESS;
   }

   public void setTimings(Timings timings)
   {
      mSavedTimings = timings;
   }
}
