package com.dataflux.architect.example;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.SealedObject;
import javax.crypto.SecretKey;

import com.dataflux.architect.BASE64;
import com.dataflux.architect.DataType;
import com.dataflux.architect.FieldInformation;
import com.dataflux.architect.ISimpleStep;
import com.dataflux.architect.StepException;

/**
 * Execute an HTTP request using the specified method ({@code GET, POST, PUT, DELETE}).
 * <p>
 * This is an example implementation of {@link com.dataflux.architect.ISimpleStep} that
 * executes the specified HTTP request.
 * </p>
 * <p>
 * BASIC authentication is supported, as well as the specification of an HTTP proxy.
 * This step could be useful for interacting with any REST-ful web service.
 * </p>
 * <p>
 * The parameters "URL Template" and "Content Template" allow input field substitution
 * by using curly braces.  For example, <code>http://my.server.com?id={ID}</code> will substitute
 * the value of the input field called "ID" into the URL used for the request.
 * </p>
 * <p>
 * <strong>Note:</strong> The plugin examples are included in the Data Management Platform Java&#x2122;
 * Plugin jar and as such have a package name that represents that location.  Therefore, modifying the
 * examples requires placing the modified source, including the package statement, in a new Java&#x2122;
 * compilation unit.
 * </p>
 * @author <a href="http://www.sas.com">SAS Institute</a>
 * @since dfPower 8.0
 */
public class HttpRequestStep
       implements ISimpleStep
{
   private int numInputFields;
   private Object[] fieldValues;
   private DataType[] inputFieldTypes;
   private String[] inputFieldNames;
   private int[] inputFieldLengths;

   private Map<String, Object> inputs = new HashMap<String, Object>();

   private String urlTemplate;
   private String httpMethod;
   private String contentTemplate;
   private String username;
   private SealedObject password;
   private String proxyHost;
   private String proxyPort;
   private String maxResponseSize;
   private char[] responseBuffer;

   private final static int DEFAULT_RESPONSE_SIZE = 256;
   private SecretKey key;
   private Cipher cipher;


   public HttpRequestStep() throws StepException {
	super();

	// Setup cipher so we can encrypt the password
    try {
		cipher = Cipher.getInstance("AES");
		KeyGenerator keyGenerator = KeyGenerator.getInstance("AES");
		keyGenerator.init(128);
		key = keyGenerator.generateKey();
		cipher.init(Cipher.ENCRYPT_MODE, key);
	    
    } catch (Exception e ) {
    	throw new StepException(e.getMessage());
    }

}

public int getOutputFieldCount() {
      return numInputFields + 2;
   }

   public FieldInformation getOutputFieldInformation(int index) throws IndexOutOfBoundsException {
      if (index == numInputFields) {
         return new FieldInformation("HTTP_RESPONSE", DataType.STRING, responseBuffer.length);
      }
      else if (index == numInputFields + 1) {
         return new FieldInformation("HTTP_STATUS", DataType.INTEGER, 0);
      }
      else {
         return new FieldInformation(inputFieldNames[index], inputFieldTypes[index], inputFieldLengths[index]);
      }
   }

   public Object getOutputFieldValue(int index) throws IndexOutOfBoundsException {
      return fieldValues[index];
   }

   public int getParameterCount() {
      return 8;
   }

   public String getParameterName(int index) throws IndexOutOfBoundsException {
      switch (index) {
         case 0:
            return "URL Template";
         case 1:
            return "HTTP Method";
         case 2:
            return "Content Template";
         case 3:
            return "User Name";
         case 4:
            return "Password";
         case 5:
            return "Proxy Host";
         case 6:
            return "Proxy Port";
         case 7:
            return "Maximum Reponse Size";
         default:
            throw new IndexOutOfBoundsException();
      }
   }

   public void postExec() {

   }

   public void preExec(boolean previewMode) throws StepException {
      if (proxyHost != null && proxyHost.trim().length() > 0) {
         System.setProperty ("http.proxyHost", proxyHost);
      }
      if (proxyPort != null && proxyPort.trim().length() > 0) {
         System.setProperty ("http.proxyPort", proxyPort);
      }
   }

   public void processRow() throws StepException {
      Reader r = null;
      Writer w = null;
      try {
         String url = processTemplate(urlTemplate, true);
         HttpURLConnection con = (HttpURLConnection) new URL(url).openConnection();
         con.setRequestMethod(httpMethod);

         if (username != null && username.trim().length() > 0) {
            String creds = BASE64.encode((username + ":" + password.getObject(key)).getBytes());
            con.setRequestProperty("Authorization", "BASIC " + creds);
         }
         if (("POST".equals(httpMethod) || "PUT".equals(httpMethod)) && contentTemplate != null) {
            con.setDoOutput(true);
            String content = processTemplate(contentTemplate, false);
            con.setRequestProperty("Content-Length", String.valueOf(content.length()));
            w = new OutputStreamWriter(con.getOutputStream());
            w.write(content);
            w.close();
         }

         r = new InputStreamReader(con.getInputStream());
         int count = r.read(responseBuffer);
         fieldValues[numInputFields] = count > 0 ? new String(responseBuffer, 0, count) : "";
         fieldValues[numInputFields + 1] = new Integer(con.getResponseCode());
      }
      catch (Exception e) {
         throw new StepException(e.getMessage());
      }
      finally {
         try { r.close(); } catch (Exception ignored) {}
         try { w.close(); } catch (Exception ignored) {}
      }
   }

   public void release() {
   }

   public void setInputFieldCount(int count) {
      inputFieldTypes = new DataType[count];
      inputFieldNames = new String[count];
      inputFieldLengths = new int[count];

      // in this sample, we are passing through all the fields plus our one additional field
      fieldValues = new Object[count + 2];

      numInputFields = count;
   }

   public void setInputFieldInformation(int index, String name, DataType type, int length) {
      inputFieldNames[index] = name;
      inputFieldTypes[index] = type;
      inputFieldLengths[index] = length;
   }

   public void setInputFieldValue(int index, Object value) {
      fieldValues[index] = value;
      inputs.put(inputFieldNames[index], value);
   }

   public void setParameterValue(int index, String value)
		   throws IndexOutOfBoundsException, IllegalBlockSizeException, IOException {
      switch (index) {
         case 0:
            urlTemplate = value;
            break;
         case 1:
            httpMethod = value.toUpperCase();
            break;
         case 2:
            contentTemplate = value;
            break;
         case 3:
            username = value;
            break;
         case 4:
            password = new SealedObject(value, cipher);
            break;
         case 5:
            proxyHost = value;
            break;
         case 6:
            proxyPort = value;
            break;
         case 7:
            maxResponseSize = value;
            break;
         default:
            throw new IndexOutOfBoundsException();
      }
   }

   public void validateInputs() throws StepException {
      try {
         responseBuffer = new char[Integer.parseInt(maxResponseSize)];
      }
      catch (Exception e) {
         responseBuffer = new char[DEFAULT_RESPONSE_SIZE];
      }

      if (("GET".equals(httpMethod) ||
            "POST".equals(httpMethod) ||
            "PUT".equals(httpMethod) ||
            "DELETE".equals(httpMethod)) == false) {
         throw new StepException("Invalid HTTP method");
      }
      if (urlTemplate == null || !urlTemplate.toLowerCase().startsWith("http://")) {
         throw new StepException("Invalid HTTP URI template");
      }
   }

   private String processTemplate(String template, boolean encode) throws UnsupportedEncodingException {
      String s = template;
      Iterator<String> i = inputs.keySet().iterator();
      while (i.hasNext()) {
         String name = (String)i.next();
         Object value = inputs.get(name);
         if (value != null) {
            String fragment = encode ? URLEncoder.encode(value.toString(), "UTF-8") : value.toString();
            s = s.replaceAll("\\{" + name + "\\}", fragment);
         }
      }
      return s;
   }
}
