package com.dataflux.architect.example;
import java.util.ArrayList;

import com.dataflux.architect.DataType;
import com.dataflux.architect.IStep;
import com.dataflux.architect.PlugStep;
import com.dataflux.architect.StepException;
import com.dataflux.architect.StepOutputItem;
import com.dataflux.architect.StepOutputItems;
import com.dataflux.architect.StepReadState;
import com.dataflux.architect.StepStatus;
import com.dataflux.architect.PrepState;
import com.dataflux.architect.StepStep;

/**
 * Generate multiple output rows per input row.
 * <p>
 * This is an example implementation of {@link com.dataflux.architect.IStep} that
 * generates multiple output rows for each input row.
 * </p>
 * <p>
 * <strong>Note:</strong> The plugin examples are included in the Data Management Platform Java&#x2122;
 * Plugin jar and as such have a package name that represents that location.  Therefore, modifying the
 * examples requires placing the modified source, including the package statement, in a new Java&#x2122;
 * compilation unit.
 * </p>
 * @author <a href="http://www.sas.com">SAS Institute</a>
 * @since dfPower Studio 8.0
 */
public class MultiRowOutputStepExample
       implements IStep
{
   PlugStep step;
   long numRowsRead;
   StepStatus status = new StepStatus();
   boolean logCreated = false;
   String fieldToProcess;
   int indexToProcess;
   String outputFieldName;

   StepOutputItem itemToProcess;
   ArrayList<String> outputValues;

   public long getNumRows() {
      StepStep stepStep = step.getStep();
      if (stepStep.getNumParents() > 0) {
         // We generate three output rows for each input row.
         return stepStep.getParent(0).getNumRows() * 3;
       }

      return -1;
   }

   public String getParameterName(int index) {
      if (index == 0) {
         return "Input field to process";
      }
      else if (index == 1) {
         return "Output field name";
      }
      throw new IndexOutOfBoundsException();
   }

   public int getParameterCount() {
      return 2;
   }

   public StepStatus getStatus() {
      StepStep stepStep = step.getStep();
      if (stepStep.getNumParents() > 0) {
         StepStep parent = stepStep.getParent(0);
         long nr = parent.getNumRows();
           if (nr > 0)
               status.setPercent((int) (numRowsRead * 100 / nr));
           else
               status.setPercent(parent.getStatus().getPercent());
       } else {
           status.setPercent(0);
      }

       status.setStatus("Processing rows");

       return status;
   }

   public void initialize(PlugStep step) throws StepException {
      this.step = step;
   }

   public PrepState prep(int flags) {
      // input requirements = output names from parent, if parent exists
       StepOutputItems parentItems = step.parentOutputItems(flags == PREP_FLAG_FULL);
       step.clearInputNames();

       indexToProcess = -1;
       if (parentItems != null) {
          int count = parentItems.getCount();
           for (int i = 0; i < count; i++) {
              StepOutputItem item = parentItems.getItem(i);
              if (item.getName().equals(fieldToProcess)) {
               indexToProcess = i;
            }
               step.addInputName(item.getName());
         }

         if (indexToProcess < 0) {
            step.setError("Could not find parent field named '" + fieldToProcess + "'");
            return PrepState.FAILED;
         }
       }

       if (flags == PREP_FLAG_QUICK) {
          return PrepState.INPUTNAMES;
       }

       StepOutputItems myItems = step.buildOutputItems(1);
       StepOutputItem item = myItems.getItem(0);
       item.setMaxCharLength(25);
       item.setMaxByteLength(item.getMaxCharLength() * 4);
       item.setType(DataType.STRING);
       item.setName(outputFieldName);

       outputValues = new ArrayList<String>();

      return PrepState.ALL_SUCCESS;
   }

   public boolean preExec() {
      numRowsRead = 0;
      return true;
   }

   public StepReadState next() {

      if (outputValues.isEmpty()) {
         StepReadState state = step.parentNext();
          if (state != StepReadState.SUCCESS) {
              return state;
         }

         numRowsRead++;

         StepOutputItems parentItems = step.parentOutputItems(true);

         try {
            itemToProcess = parentItems.getItem(indexToProcess);
         }
         catch (IndexOutOfBoundsException e) {
            step.setError("Could not find parent field named '" + fieldToProcess + "'");
            return StepReadState.ERROR;
         }

         Object value = itemToProcess.getValue();
         outputValues.add(value + " [ONE]");
         outputValues.add(value + " [TWO]");
         outputValues.add(value + " [THREE]");
      }

      StepOutputItems myItems = step.getExistingOutputItems();
      String outputValue = (String) outputValues.remove(0);
      myItems.getItem(0).setValue(outputValue);

      return StepReadState.SUCCESS;
   }

   public void postExec() {
      step.appendStats(numRowsRead + " row(s) processed.", false);
   }

   public void release() {
      this.step = null;
   }

   public void setParameterValue(int index, String value) {
      if (index == 0) {
         this.fieldToProcess = value;
      }
      else if (index == 1) {
         outputFieldName = value;
      }
      else {
         throw new IndexOutOfBoundsException();
      }
   }
}
