package com.dataflux.architect.example;

import com.dataflux.architect.DataType;
import com.dataflux.architect.PrepState;
import com.dataflux.architect.StepAdapter;
import com.dataflux.architect.StepOutputItem;
import com.dataflux.architect.StepOutputItems;
import com.dataflux.architect.StepReadState;
import java.util.HashMap;

/**
 * Demonstrate {@link com.dataflux.architect.IStep} semantics.
 * <p>
 * This is an example implementation of
 * {@link com.dataflux.architect.IStep}, through extension of the
 * {@link com.dataflux.architect.StepAdapter}, that appends the
 * value of an input field to the string <code>"Hello to "</code> and
 * places the result in a new field.
 * </p>
 * <p>
 * <strong>Note:</strong> The plugin examples are included in the Data Management Platform Java&#x2122;
 * Plugin jar and as such have a package name that represents that location.  Therefore, modifying the
 * examples requires placing the modified source, including the package statement, in a new Java&#x2122;
 * compilation unit.
 * </p>
 * @author <a href="http://www.sas.com">SAS Institute</a>
 * @since dfPower Studio 8.0
 */
public class StepExample
       extends StepAdapter
{
   /** Index of the parent item containing the field to process. */
   int mIndexToProcess;

   /**
    * Parameter associated with the name of the input field in which to manipulate.
    */
   static final String FIELD_TO_PROCESS = "Input field to process";

   /**
    * Parameter associated with the name of the new output field.
    */
   static final String OUTPUT_FIELD = "Output field name";

   /**
    * Constructor.
    * <p>
    * This example requires two parameters.  To enable parameter processing by the
    * {@link com.dataflux.architect.StepAdapter} the constructor must
    * create/populate {@link com.dataflux.architect.StepAdapter#mParameterNames mParameterNames}
    * and {@link com.dataflux.architect.StepAdapter#mParameterValues mParameterValues}.
    * </p>
    */
   public StepExample()
   {
      mParameterValues   = new HashMap<String, String>();
      mParameterNames    = new String[2];
      mParameterNames[0] = FIELD_TO_PROCESS;
      mParameterNames[1] = OUTPUT_FIELD;
   }

   public StepReadState next()
   {
      /**
       * Obtain the next row of input from our parent step.
       *
       * If an error or end of file is encountered, return early.  Otherwise,
       * count the read row.
       */
      StepReadState state = step.parentNext();
      if (StepReadState.SUCCESS != state)
         return state;
      mNumRowsRead++;

      /**
       * Obtain the set of fields in the current row.
       *
       * From there, we'll gain access to the field to process.
       */
      StepOutputItems parentItems   = step.parentOutputItems(true);
      StepOutputItem  itemToProcess = null;
      try
         {
         itemToProcess = parentItems.getItem(mIndexToProcess);
         }
      catch (IndexOutOfBoundsException e)
         {
         step.setError("Could not find parent field named '" + mParameterValues.get(FIELD_TO_PROCESS) + "'.");
         return StepReadState.ERROR;
         }

      // Set the value of the sole output field using the input field as the suffix.
      StepOutputItems myItems = step.getExistingOutputItems();
      myItems.getItem(0).setValue("Hello to " + itemToProcess.getValue());

      return StepReadState.SUCCESS;
   }

   public boolean preExec()
   {
      super.preExec();

      // This example requires a parent step
      if (0 == step.getStep().getNumParents())
         {
         step.setError(this.getClass().getSimpleName() + " requires a parent step.");
         return false;
         }

      return true;
   }

   public PrepState prep(int flags)
   {
      /**
       * prep() may be called multiple times with differing parent step fields.
       * Therefore, the index to the field in which to process needs to be located
       * each time.
       */
      String fieldToProcess = mParameterValues.get(FIELD_TO_PROCESS);
      mIndexToProcess       = -1;

      /**
       * prep() needs to ensure the output items from the parent step meet the
       * needs of this step.  And, if this step produces output items, the set
       * of output items needs to be communicated to the Step Engine.
       *
       * Start by clearing out the input field requirements.
       */
      step.clearInputNames();

      /**
       * This example uses the parent step output field identified by the
       * {@link #FIELD_TO_PROCESS} parameter as a suffix.  Therefore, the
       * parent step output field needs to exist and its index needs to be
       * remembered.
       */
      StepOutputItems parentItems = step.parentOutputItems(flags == PREP_FLAG_FULL);
      if (null != parentItems)
         {
         int count = parentItems.getCount();

         // Locate the field to process and remember its item index
         for (int i = 0; i < count; i++)
            {
            StepOutputItem item = parentItems.getItem(i);
            if (item.getName().equals(fieldToProcess))
               mIndexToProcess = i;
            step.addInputName(item.getName());
            }

         if (mIndexToProcess < 0)
            {
            step.setError("Could not find parent field named '" + fieldToProcess + "'");
            return PrepState.FAILED;
            }
         }

      // If a quick prep(), then there is no need to return our output fields.
      if (flags == PREP_FLAG_QUICK)
         return PrepState.INPUTNAMES;

      /**
       * Caller requested a full prep().  Therefore the step must populate its set of
       * output fields.
       *
       * In this example there is only one output field and the name of the output field
       * was specified in the {@link #OUTPUT_FIELD} parameter.
       */
      StepOutputItems myItems = step.buildOutputItems(1);
      StepOutputItem  item    = myItems.getItem(0);
      item.setMaxCharLength(25);
      item.setMaxByteLength(item.getMaxCharLength() * 4);
      item.setType(DataType.STRING);
      item.setName(mParameterValues.get(OUTPUT_FIELD));

      return PrepState.ALL_SUCCESS;
   }
}
